
#line 1 "src/gpg-protocol.rl"
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) Simon Gomizelj, 2015
 */

#include "gpg-protocol.h"

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <err.h>
#include <errno.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/un.h>

#include "util.h"

struct gpg_t {
    int fd;
    char buf[BUFSIZ];

    /* ragel parser state */
    int cs;
    char *p;
    char *pe;
};

static int gpg_buffer_refill(struct gpg_t *gpg)
{
    ssize_t nbytes_r = read(gpg->fd, gpg->buf, BUFSIZ);
    if (nbytes_r < 0)
        return -1;

    gpg->buf[nbytes_r] = 0;
    gpg->p = gpg->buf;
    gpg->pe = gpg->buf + nbytes_r;
    return nbytes_r;
}


#line 65 "src/gpg-protocol.rl"



#line 62 "src/gpg-protocol.c"
static const char _status_actions[] = {
	0, 1, 0, 1, 1
};

static const char _status_trans_keys[] = {
	0, 0, 69, 79, 82, 82, 82, 82, 10, 10, 75, 75, 0, 0, 0
};

static const char _status_key_spans[] = {
	0, 11, 1, 1, 1, 1, 0
};

static const char _status_index_offsets[] = {
	0, 0, 12, 14, 16, 18, 20
};

static const char _status_indicies[] = {
	0, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 2, 1, 3, 1, 4, 
	1, 5, 4, 4, 1, 1, 0
};

static const char _status_trans_targs[] = {
	2, 0, 5, 3, 4, 6
};

static const char _status_trans_actions[] = {
	1, 0, 0, 0, 0, 0
};

static const char _status_eof_actions[] = {
	0, 0, 0, 0, 0, 0, 3
};

static const int status_start = 1;
static const int status_error = 0;

static const int status_en_main = 1;


#line 68 "src/gpg-protocol.rl"

static int gpg_check_return(struct gpg_t *gpg)
{
    int rc = 0;

    
#line 74 "src/gpg-protocol.rl"
    
#line 75 "src/gpg-protocol.rl"
    
#line 76 "src/gpg-protocol.rl"
    
#line 116 "src/gpg-protocol.c"
	{
	 gpg->cs = status_start;
	}

#line 77 "src/gpg-protocol.rl"

    for (;;) {
        if (gpg->p == NULL || gpg->p == gpg->pe) {
            if (gpg_buffer_refill(gpg) <= 0)
                break;
        }

        char *eof = gpg->pe;
        
#line 131 "src/gpg-protocol.c"
	{
	int _slen;
	int _trans;
	const char *_acts;
	unsigned int _nacts;
	const char *_keys;
	const char *_inds;

	if ( ( gpg->p) == ( gpg->pe) )
		goto _test_eof;
	if (  gpg->cs == 0 )
		goto _out;
_resume:
	_keys = _status_trans_keys + ( gpg->cs<<1);
	_inds = _status_indicies + _status_index_offsets[ gpg->cs];

	_slen = _status_key_spans[ gpg->cs];
	_trans = _inds[ _slen > 0 && _keys[0] <=(*( gpg->p)) &&
		(*( gpg->p)) <= _keys[1] ?
		(*( gpg->p)) - _keys[0] : _slen ];

	 gpg->cs = _status_trans_targs[_trans];

	if ( _status_trans_actions[_trans] == 0 )
		goto _again;

	_acts = _status_actions + _status_trans_actions[_trans];
	_nacts = (unsigned int) *_acts++;
	while ( _nacts-- > 0 ) {
		switch ( *(_acts++) )
		{
	case 0:
#line 57 "src/gpg-protocol.rl"
	{
        fprintf(stderr, "%s: gpg protocol error: %s", program_invocation_short_name, ( gpg->p));
        rc = -EIO;
    }
	break;
#line 170 "src/gpg-protocol.c"
		}
	}

_again:
	if (  gpg->cs == 0 )
		goto _out;
	if ( ++( gpg->p) != ( gpg->pe) )
		goto _resume;
	_test_eof: {}
	if ( ( gpg->p) == eof )
	{
	const char *__acts = _status_actions + _status_eof_actions[ gpg->cs];
	unsigned int __nacts = (unsigned int) *__acts++;
	while ( __nacts-- > 0 ) {
		switch ( *__acts++ ) {
	case 1:
#line 61 "src/gpg-protocol.rl"
	{ return rc; }
	break;
#line 190 "src/gpg-protocol.c"
		}
	}
	}

	_out: {}
	}

#line 86 "src/gpg-protocol.rl"

        if (gpg->cs == status_error) {
            warnx("error parsing gpg protocol");
            break;
        }
    }

    (void)status_en_main;
    return rc;
}

static _printf_(2, 3) int gpg_send_message(struct gpg_t *gpg, const char *fmt, ...)
{
    va_list ap;
    int nbytes_r, rc;

    va_start(ap, fmt);
    nbytes_r = vdprintf(gpg->fd, fmt, ap);
    va_end(ap);

    rc = gpg_check_return(gpg);
    return rc == 0 ? nbytes_r : rc;
}

struct gpg_t *gpg_agent_connection(const char *sock, const char *home)
{
    union {
        struct sockaddr sa;
        struct sockaddr_un un;
    } sa;
    size_t len;
    socklen_t sa_len;

    int fd = socket(AF_UNIX, SOCK_STREAM, 0);
    if (fd < 0)
        return NULL;

    sa.un = (struct sockaddr_un){ .sun_family = AF_UNIX };

    if (!sock || !sock[0]) {
        len = snprintf(sa.un.sun_path, sizeof(sa.un.sun_path),
                       "%s/.gnupg/S.gpg-agent",
                       home ? home : get_home_dir());
    } else {
        len = strcspn(sock, ":");
        memcpy(&sa.un.sun_path, sock, len);
    }

    sa_len = len + sizeof(sa.un.sun_family);
    if (connect(fd, &sa.sa, sa_len) < 0)
        return NULL;

    struct gpg_t *gpg = malloc(sizeof(struct gpg_t));
    *gpg = (struct gpg_t) { .fd = fd };

    if (gpg_check_return(gpg) < 0) {
        gpg_close(gpg);
        return NULL;
    }

    return gpg;
}

int gpg_reload_agent(struct gpg_t *gpg)
{
    return gpg_send_message(gpg, "RELOADAGENT\n");
}

int gpg_update_tty(struct gpg_t *gpg)
{
    const char *tty = ttyname(STDIN_FILENO);
    const char *term = getenv("TERM");
    const char *display = getenv("DISPLAY");
    const char *xauthority = getenv("XAUTHORITY");

    /* In this case, there's no information to update, so lets bail to
     * avoid clearing what's already there */
    if (!tty && !display)
        return 0;

    gpg_send_message(gpg, "RESET\n");

    if (tty) {
        gpg_send_message(gpg, "OPTION ttyname=%s\n", tty);
        gpg_send_message(gpg, "OPTION ttytype=%s\n", term ? term : "dumb");
    }

    if (display) {
        gpg_send_message(gpg, "OPTION display=%s\n", display);

        if (xauthority)
            gpg_send_message(gpg, "OPTION xauthority=%s\n", xauthority);
        else
            gpg_send_message(gpg, "OPTION xauthority=%s/.Xauthority\n", get_home_dir());
    }

    gpg_send_message(gpg, "UPDATESTARTUPTTY\n");
    return 0;
}


#line 272 "src/gpg-protocol.rl"



#line 304 "src/gpg-protocol.c"
static const char _keyinfo_actions[] = {
	0, 1, 1, 1, 2, 1, 3, 1, 
	4, 1, 5, 2, 0, 1, 2, 3, 
	5, 2, 5, 4
};

static const char _keyinfo_trans_keys[] = {
	0, 0, 82, 82, 82, 82, 10, 10, 75, 75, 10, 10, 32, 32, 75, 75, 
	69, 69, 89, 89, 73, 73, 78, 78, 70, 70, 79, 79, 9, 32, 48, 102, 
	48, 102, 48, 102, 48, 102, 48, 102, 48, 102, 48, 102, 48, 102, 48, 102, 
	48, 102, 48, 102, 48, 102, 48, 102, 48, 102, 48, 102, 48, 102, 48, 102, 
	48, 102, 48, 102, 48, 102, 48, 102, 48, 102, 48, 102, 48, 102, 48, 102, 
	48, 102, 48, 102, 48, 102, 48, 102, 48, 102, 48, 102, 48, 102, 48, 102, 
	48, 102, 48, 102, 48, 102, 48, 102, 48, 102, 48, 102, 48, 102, 9, 32, 
	45, 88, 9, 32, 45, 122, 9, 32, 32, 32, 9, 32, 9, 49, 45, 49, 
	9, 32, 45, 80, 9, 32, 45, 45, 9, 32, 45, 57, 9, 32, 45, 99, 
	10, 99, 9, 57, 9, 32, 9, 80, 9, 32, 9, 80, 9, 32, 9, 80, 
	9, 32, 9, 99, 9, 99, 9, 57, 9, 99, 9, 99, 9, 32, 9, 49, 
	9, 32, 9, 80, 9, 32, 9, 99, 9, 99, 9, 57, 9, 99, 9, 99, 
	9, 82, 9, 82, 9, 32, 9, 49, 9, 75, 9, 32, 9, 32, 45, 75, 
	10, 49, 9, 32, 10, 80, 9, 32, 10, 45, 9, 32, 10, 57, 9, 32, 
	10, 99, 10, 99, 10, 99, 9, 57, 9, 32, 9, 80, 9, 32, 9, 80, 
	9, 32, 9, 80, 9, 32, 9, 99, 9, 99, 9, 99, 9, 99, 10, 99, 
	9, 99, 10, 99, 9, 99, 10, 99, 9, 99, 10, 99, 9, 99, 9, 99, 
	9, 99, 45, 99, 9, 99, 45, 99, 9, 57, 10, 99, 9, 99, 9, 99, 
	9, 99, 9, 57, 9, 99, 9, 99, 10, 80, 9, 32, 10, 80, 9, 32, 
	10, 57, 9, 32, 9, 32, 9, 32, 45, 57, 10, 99, 9, 99, 9, 57, 
	9, 99, 9, 99, 10, 80, 9, 32, 10, 80, 9, 32, 9, 32, 9, 32, 
	9, 49, 9, 32, 9, 80, 9, 32, 9, 99, 9, 99, 10, 80, 9, 32, 
	10, 99, 9, 99, 10, 80, 9, 32, 10, 99, 9, 99, 9, 99, 10, 49, 
	9, 32, 10, 80, 9, 32, 10, 99, 9, 99, 9, 99, 9, 32, 45, 99, 
	10, 99, 9, 99, 45, 80, 9, 32, 45, 80, 45, 99, 9, 99, 45, 80, 
	45, 80, 9, 32, 45, 99, 9, 57, 45, 99, 45, 99, 9, 99, 45, 80, 
	9, 32, 45, 99, 45, 49, 9, 32, 45, 80, 45, 80, 9, 32, 45, 99, 
	9, 99, 45, 99, 9, 99, 9, 122, 69, 83, 69, 83, 9, 99, 9, 83, 
	9, 83, 9, 83, 9, 83, 45, 99, 45, 83, 45, 83, 45, 83, 9, 99, 
	9, 83, 45, 99, 45, 99, 45, 99, 45, 99, 45, 99, 45, 99, 45, 99, 
	45, 83, 9, 83, 45, 83, 45, 83, 9, 99, 9, 83, 45, 83, 9, 83, 
	9, 99, 9, 83, 45, 99, 45, 83, 9, 83, 45, 99, 45, 83, 45, 83, 
	9, 99, 45, 83, 45, 83, 45, 83, 45, 99, 0
};

static const char _keyinfo_key_spans[] = {
	0, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 24, 55, 
	55, 55, 55, 55, 55, 55, 55, 55, 
	55, 55, 55, 55, 55, 55, 55, 55, 
	55, 55, 55, 55, 55, 55, 55, 55, 
	55, 55, 55, 55, 55, 55, 55, 55, 
	55, 55, 55, 55, 55, 55, 55, 24, 
	44, 24, 78, 24, 1, 24, 41, 5, 
	24, 36, 24, 1, 24, 13, 24, 55, 
	90, 49, 24, 72, 24, 72, 24, 72, 
	24, 91, 91, 49, 91, 91, 24, 41, 
	24, 72, 24, 91, 91, 49, 91, 91, 
	74, 74, 24, 41, 67, 24, 24, 31, 
	40, 24, 71, 24, 36, 24, 48, 24, 
	90, 90, 90, 49, 24, 72, 24, 72, 
	24, 72, 24, 91, 91, 91, 91, 90, 
	91, 90, 91, 90, 91, 90, 91, 91, 
	91, 55, 91, 55, 49, 90, 91, 91, 
	91, 49, 91, 91, 71, 24, 71, 24, 
	48, 24, 24, 24, 13, 90, 91, 49, 
	91, 91, 71, 24, 71, 24, 24, 24, 
	41, 24, 72, 24, 91, 91, 71, 24, 
	90, 91, 71, 24, 90, 91, 91, 40, 
	24, 71, 24, 90, 91, 91, 24, 55, 
	90, 91, 36, 24, 36, 55, 91, 36, 
	36, 24, 55, 49, 55, 55, 91, 36, 
	24, 55, 5, 24, 36, 36, 24, 55, 
	91, 55, 91, 114, 15, 15, 91, 75, 
	75, 75, 75, 55, 39, 39, 39, 91, 
	75, 55, 55, 55, 55, 55, 55, 55, 
	39, 75, 39, 39, 91, 75, 39, 75, 
	91, 75, 55, 39, 75, 55, 39, 39, 
	91, 39, 39, 39, 55
};

static const short _keyinfo_index_offsets[] = {
	0, 0, 2, 4, 6, 8, 10, 12, 
	14, 16, 18, 20, 22, 24, 26, 51, 
	107, 163, 219, 275, 331, 387, 443, 499, 
	555, 611, 667, 723, 779, 835, 891, 947, 
	1003, 1059, 1115, 1171, 1227, 1283, 1339, 1395, 
	1451, 1507, 1563, 1619, 1675, 1731, 1787, 1843, 
	1899, 1955, 2011, 2067, 2123, 2179, 2235, 2291, 
	2316, 2361, 2386, 2465, 2490, 2492, 2517, 2559, 
	2565, 2590, 2627, 2652, 2654, 2679, 2693, 2718, 
	2774, 2865, 2915, 2940, 3013, 3038, 3111, 3136, 
	3209, 3234, 3326, 3418, 3468, 3560, 3652, 3677, 
	3719, 3744, 3817, 3842, 3934, 4026, 4076, 4168, 
	4260, 4335, 4410, 4435, 4477, 4545, 4570, 4595, 
	4627, 4668, 4693, 4765, 4790, 4827, 4852, 4901, 
	4926, 5017, 5108, 5199, 5249, 5274, 5347, 5372, 
	5445, 5470, 5543, 5568, 5660, 5752, 5844, 5936, 
	6027, 6119, 6210, 6302, 6393, 6485, 6576, 6668, 
	6760, 6852, 6908, 7000, 7056, 7106, 7197, 7289, 
	7381, 7473, 7523, 7615, 7707, 7779, 7804, 7876, 
	7901, 7950, 7975, 8000, 8025, 8039, 8130, 8222, 
	8272, 8364, 8456, 8528, 8553, 8625, 8650, 8675, 
	8700, 8742, 8767, 8840, 8865, 8957, 9049, 9121, 
	9146, 9237, 9329, 9401, 9426, 9517, 9609, 9701, 
	9742, 9767, 9839, 9864, 9955, 10047, 10139, 10164, 
	10220, 10311, 10403, 10440, 10465, 10502, 10558, 10650, 
	10687, 10724, 10749, 10805, 10855, 10911, 10967, 11059, 
	11096, 11121, 11177, 11183, 11208, 11245, 11282, 11307, 
	11363, 11455, 11511, 11603, 11718, 11734, 11750, 11842, 
	11918, 11994, 12070, 12146, 12202, 12242, 12282, 12322, 
	12414, 12490, 12546, 12602, 12658, 12714, 12770, 12826, 
	12882, 12922, 12998, 13038, 13078, 13170, 13246, 13286, 
	13362, 13454, 13530, 13586, 13626, 13702, 13758, 13798, 
	13838, 13930, 13970, 14010, 14050
};

static const short _keyinfo_indicies[] = {
	0, 1, 2, 1, 3, 2, 4, 
	1, 3, 1, 5, 1, 6, 1, 7, 
	1, 8, 1, 9, 1, 10, 1, 11, 
	1, 12, 1, 13, 13, 13, 13, 13, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 13, 1, 14, 14, 14, 14, 
	14, 14, 14, 14, 14, 14, 1, 1, 
	1, 1, 1, 1, 1, 14, 14, 14, 
	14, 14, 14, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 14, 14, 14, 
	14, 14, 14, 1, 15, 15, 15, 15, 
	15, 15, 15, 15, 15, 15, 1, 1, 
	1, 1, 1, 1, 1, 15, 15, 15, 
	15, 15, 15, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 15, 15, 15, 
	15, 15, 15, 1, 16, 16, 16, 16, 
	16, 16, 16, 16, 16, 16, 1, 1, 
	1, 1, 1, 1, 1, 16, 16, 16, 
	16, 16, 16, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 16, 16, 16, 
	16, 16, 16, 1, 17, 17, 17, 17, 
	17, 17, 17, 17, 17, 17, 1, 1, 
	1, 1, 1, 1, 1, 17, 17, 17, 
	17, 17, 17, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 17, 17, 17, 
	17, 17, 17, 1, 18, 18, 18, 18, 
	18, 18, 18, 18, 18, 18, 1, 1, 
	1, 1, 1, 1, 1, 18, 18, 18, 
	18, 18, 18, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 18, 18, 18, 
	18, 18, 18, 1, 19, 19, 19, 19, 
	19, 19, 19, 19, 19, 19, 1, 1, 
	1, 1, 1, 1, 1, 19, 19, 19, 
	19, 19, 19, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 19, 19, 19, 
	19, 19, 19, 1, 20, 20, 20, 20, 
	20, 20, 20, 20, 20, 20, 1, 1, 
	1, 1, 1, 1, 1, 20, 20, 20, 
	20, 20, 20, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 20, 20, 20, 
	20, 20, 20, 1, 21, 21, 21, 21, 
	21, 21, 21, 21, 21, 21, 1, 1, 
	1, 1, 1, 1, 1, 21, 21, 21, 
	21, 21, 21, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 21, 21, 21, 
	21, 21, 21, 1, 22, 22, 22, 22, 
	22, 22, 22, 22, 22, 22, 1, 1, 
	1, 1, 1, 1, 1, 22, 22, 22, 
	22, 22, 22, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 22, 22, 22, 
	22, 22, 22, 1, 23, 23, 23, 23, 
	23, 23, 23, 23, 23, 23, 1, 1, 
	1, 1, 1, 1, 1, 23, 23, 23, 
	23, 23, 23, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 23, 23, 23, 
	23, 23, 23, 1, 24, 24, 24, 24, 
	24, 24, 24, 24, 24, 24, 1, 1, 
	1, 1, 1, 1, 1, 24, 24, 24, 
	24, 24, 24, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 24, 24, 24, 
	24, 24, 24, 1, 25, 25, 25, 25, 
	25, 25, 25, 25, 25, 25, 1, 1, 
	1, 1, 1, 1, 1, 25, 25, 25, 
	25, 25, 25, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 25, 25, 25, 
	25, 25, 25, 1, 26, 26, 26, 26, 
	26, 26, 26, 26, 26, 26, 1, 1, 
	1, 1, 1, 1, 1, 26, 26, 26, 
	26, 26, 26, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 26, 26, 26, 
	26, 26, 26, 1, 27, 27, 27, 27, 
	27, 27, 27, 27, 27, 27, 1, 1, 
	1, 1, 1, 1, 1, 27, 27, 27, 
	27, 27, 27, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 27, 27, 27, 
	27, 27, 27, 1, 28, 28, 28, 28, 
	28, 28, 28, 28, 28, 28, 1, 1, 
	1, 1, 1, 1, 1, 28, 28, 28, 
	28, 28, 28, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 28, 28, 28, 
	28, 28, 28, 1, 29, 29, 29, 29, 
	29, 29, 29, 29, 29, 29, 1, 1, 
	1, 1, 1, 1, 1, 29, 29, 29, 
	29, 29, 29, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 29, 29, 29, 
	29, 29, 29, 1, 30, 30, 30, 30, 
	30, 30, 30, 30, 30, 30, 1, 1, 
	1, 1, 1, 1, 1, 30, 30, 30, 
	30, 30, 30, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 30, 30, 30, 
	30, 30, 30, 1, 31, 31, 31, 31, 
	31, 31, 31, 31, 31, 31, 1, 1, 
	1, 1, 1, 1, 1, 31, 31, 31, 
	31, 31, 31, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 31, 31, 31, 
	31, 31, 31, 1, 32, 32, 32, 32, 
	32, 32, 32, 32, 32, 32, 1, 1, 
	1, 1, 1, 1, 1, 32, 32, 32, 
	32, 32, 32, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 32, 32, 32, 
	32, 32, 32, 1, 33, 33, 33, 33, 
	33, 33, 33, 33, 33, 33, 1, 1, 
	1, 1, 1, 1, 1, 33, 33, 33, 
	33, 33, 33, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 33, 33, 33, 
	33, 33, 33, 1, 34, 34, 34, 34, 
	34, 34, 34, 34, 34, 34, 1, 1, 
	1, 1, 1, 1, 1, 34, 34, 34, 
	34, 34, 34, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 34, 34, 34, 
	34, 34, 34, 1, 35, 35, 35, 35, 
	35, 35, 35, 35, 35, 35, 1, 1, 
	1, 1, 1, 1, 1, 35, 35, 35, 
	35, 35, 35, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 35, 35, 35, 
	35, 35, 35, 1, 36, 36, 36, 36, 
	36, 36, 36, 36, 36, 36, 1, 1, 
	1, 1, 1, 1, 1, 36, 36, 36, 
	36, 36, 36, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 36, 36, 36, 
	36, 36, 36, 1, 37, 37, 37, 37, 
	37, 37, 37, 37, 37, 37, 1, 1, 
	1, 1, 1, 1, 1, 37, 37, 37, 
	37, 37, 37, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 37, 37, 37, 
	37, 37, 37, 1, 38, 38, 38, 38, 
	38, 38, 38, 38, 38, 38, 1, 1, 
	1, 1, 1, 1, 1, 38, 38, 38, 
	38, 38, 38, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 38, 38, 38, 
	38, 38, 38, 1, 39, 39, 39, 39, 
	39, 39, 39, 39, 39, 39, 1, 1, 
	1, 1, 1, 1, 1, 39, 39, 39, 
	39, 39, 39, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 39, 39, 39, 
	39, 39, 39, 1, 40, 40, 40, 40, 
	40, 40, 40, 40, 40, 40, 1, 1, 
	1, 1, 1, 1, 1, 40, 40, 40, 
	40, 40, 40, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 40, 40, 40, 
	40, 40, 40, 1, 41, 41, 41, 41, 
	41, 41, 41, 41, 41, 41, 1, 1, 
	1, 1, 1, 1, 1, 41, 41, 41, 
	41, 41, 41, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 41, 41, 41, 
	41, 41, 41, 1, 42, 42, 42, 42, 
	42, 42, 42, 42, 42, 42, 1, 1, 
	1, 1, 1, 1, 1, 42, 42, 42, 
	42, 42, 42, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 42, 42, 42, 
	42, 42, 42, 1, 43, 43, 43, 43, 
	43, 43, 43, 43, 43, 43, 1, 1, 
	1, 1, 1, 1, 1, 43, 43, 43, 
	43, 43, 43, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 43, 43, 43, 
	43, 43, 43, 1, 44, 44, 44, 44, 
	44, 44, 44, 44, 44, 44, 1, 1, 
	1, 1, 1, 1, 1, 44, 44, 44, 
	44, 44, 44, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 44, 44, 44, 
	44, 44, 44, 1, 45, 45, 45, 45, 
	45, 45, 45, 45, 45, 45, 1, 1, 
	1, 1, 1, 1, 1, 45, 45, 45, 
	45, 45, 45, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 45, 45, 45, 
	45, 45, 45, 1, 46, 46, 46, 46, 
	46, 46, 46, 46, 46, 46, 1, 1, 
	1, 1, 1, 1, 1, 46, 46, 46, 
	46, 46, 46, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 46, 46, 46, 
	46, 46, 46, 1, 47, 47, 47, 47, 
	47, 47, 47, 47, 47, 47, 1, 1, 
	1, 1, 1, 1, 1, 47, 47, 47, 
	47, 47, 47, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 47, 47, 47, 
	47, 47, 47, 1, 48, 48, 48, 48, 
	48, 48, 48, 48, 48, 48, 1, 1, 
	1, 1, 1, 1, 1, 48, 48, 48, 
	48, 48, 48, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 48, 48, 48, 
	48, 48, 48, 1, 49, 49, 49, 49, 
	49, 49, 49, 49, 49, 49, 1, 1, 
	1, 1, 1, 1, 1, 49, 49, 49, 
	49, 49, 49, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 49, 49, 49, 
	49, 49, 49, 1, 50, 50, 50, 50, 
	50, 50, 50, 50, 50, 50, 1, 1, 
	1, 1, 1, 1, 1, 50, 50, 50, 
	50, 50, 50, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 50, 50, 50, 
	50, 50, 50, 1, 51, 51, 51, 51, 
	51, 51, 51, 51, 51, 51, 1, 1, 
	1, 1, 1, 1, 1, 51, 51, 51, 
	51, 51, 51, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 51, 51, 51, 
	51, 51, 51, 1, 52, 52, 52, 52, 
	52, 52, 52, 52, 52, 52, 1, 1, 
	1, 1, 1, 1, 1, 52, 52, 52, 
	52, 52, 52, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 52, 52, 52, 
	52, 52, 52, 1, 53, 53, 53, 53, 
	53, 53, 53, 53, 53, 53, 1, 1, 
	1, 1, 1, 1, 1, 53, 53, 53, 
	53, 53, 53, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 53, 53, 53, 
	53, 53, 53, 1, 54, 54, 54, 54, 
	54, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 54, 1, 55, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 55, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 55, 1, 1, 1, 
	55, 1, 56, 56, 56, 56, 56, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 56, 1, 57, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 58, 
	58, 58, 58, 58, 58, 58, 58, 58, 
	58, 58, 58, 58, 58, 58, 58, 58, 
	58, 58, 58, 58, 58, 58, 58, 58, 
	58, 1, 1, 1, 1, 1, 1, 58, 
	58, 58, 58, 58, 58, 58, 58, 58, 
	58, 58, 58, 58, 58, 58, 58, 58, 
	58, 58, 58, 58, 58, 58, 58, 58, 
	58, 1, 59, 59, 59, 59, 59, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 59, 1, 1, 60, 61, 61, 61, 
	61, 61, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 62, 60, 61, 61, 
	61, 61, 61, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 62, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 63, 60, 60, 60, 63, 60, 
	64, 1, 1, 1, 64, 1, 65, 65, 
	65, 65, 65, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 65, 1, 66, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 66, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 66, 1, 67, 67, 67, 67, 
	67, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 67, 1, 68, 1, 69, 
	69, 69, 69, 69, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 69, 1, 
	70, 1, 1, 71, 71, 71, 71, 71, 
	71, 71, 71, 71, 71, 1, 72, 72, 
	72, 72, 72, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 72, 1, 73, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 73, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 73, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 73, 1, 74, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 75, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 75, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	75, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	75, 1, 72, 72, 72, 72, 72, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 72, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 71, 71, 71, 71, 71, 71, 71, 
	71, 71, 71, 1, 76, 76, 76, 76, 
	76, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 77, 60, 61, 61, 61, 
	61, 61, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 62, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 78, 60, 60, 60, 63, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 79, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 79, 60, 80, 80, 
	80, 80, 80, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 81, 60, 61, 
	61, 61, 61, 61, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 62, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 82, 60, 60, 60, 63, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 79, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 79, 60, 
	83, 83, 83, 83, 83, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 84, 
	60, 61, 61, 61, 61, 61, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	62, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 85, 60, 60, 
	86, 87, 86, 86, 86, 86, 86, 86, 
	86, 86, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 79, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	79, 60, 88, 88, 88, 88, 88, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 89, 60, 61, 61, 61, 61, 61, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 62, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 90, 
	60, 60, 86, 87, 86, 86, 86, 86, 
	86, 86, 86, 86, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 79, 91, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 79, 60, 60, 91, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 91, 60, 88, 
	92, 88, 88, 88, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 89, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 93, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 93, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 93, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 93, 60, 94, 94, 94, 94, 94, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 95, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 86, 86, 86, 86, 86, 86, 
	86, 86, 86, 86, 60, 61, 61, 61, 
	61, 61, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 62, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 96, 60, 60, 60, 63, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	91, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 91, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 91, 
	60, 76, 97, 76, 76, 76, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	77, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 93, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 93, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 93, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 93, 60, 98, 98, 98, 
	98, 98, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 99, 60, 61, 61, 
	61, 61, 61, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 62, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 100, 60, 60, 60, 63, 60, 
	101, 101, 101, 101, 101, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 102, 
	60, 61, 61, 61, 61, 61, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	62, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 103, 60, 60, 
	86, 87, 86, 86, 86, 86, 86, 86, 
	86, 86, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 79, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	79, 60, 104, 104, 104, 104, 104, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 105, 60, 61, 61, 61, 61, 61, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 62, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 106, 
	60, 60, 60, 63, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 79, 91, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 79, 60, 60, 91, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 91, 60, 83, 
	107, 83, 83, 83, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 84, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 93, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 93, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 93, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 93, 60, 108, 108, 108, 108, 108, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 109, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 86, 86, 86, 86, 86, 86, 
	86, 86, 86, 86, 60, 61, 61, 61, 
	61, 61, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 62, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 110, 60, 60, 60, 63, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 79, 
	91, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 79, 60, 60, 91, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 91, 
	60, 80, 111, 80, 80, 80, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	81, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 93, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 93, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 93, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 93, 60, 61, 61, 61, 
	61, 61, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 62, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 112, 60, 
	61, 61, 61, 61, 61, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 62, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 113, 60, 114, 115, 114, 114, 114, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 116, 113, 114, 115, 114, 114, 
	114, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 116, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	117, 113, 113, 113, 117, 113, 61, 61, 
	61, 61, 61, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 62, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	118, 60, 61, 115, 61, 61, 61, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 62, 60, 61, 61, 61, 61, 61, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 119, 60, 64, 1, 1, 1, 
	64, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 6, 1, 3, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 120, 
	2, 2, 2, 120, 2, 121, 122, 121, 
	121, 121, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 121, 2, 3, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 123, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 123, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 123, 2, 124, 125, 
	124, 124, 124, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 124, 2, 3, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 126, 2, 127, 128, 127, 127, 
	127, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 127, 2, 3, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	129, 2, 2, 130, 130, 130, 130, 130, 
	130, 130, 130, 130, 130, 2, 131, 132, 
	131, 131, 131, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 131, 2, 3, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 133, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 133, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	133, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	133, 2, 134, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 135, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 135, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 135, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 135, 2, 74, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 5, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	75, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 75, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 75, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 75, 1, 
	131, 132, 131, 131, 131, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 131, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 130, 
	130, 130, 130, 130, 130, 130, 130, 130, 
	130, 2, 136, 137, 136, 136, 136, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 138, 113, 114, 115, 114, 114, 114, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 116, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 139, 
	113, 113, 113, 117, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 140, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 140, 113, 141, 142, 141, 141, 
	141, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 143, 113, 114, 115, 114, 
	114, 114, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 116, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 144, 113, 113, 113, 117, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 140, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 140, 113, 145, 146, 
	145, 145, 145, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 147, 113, 114, 
	115, 114, 114, 114, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 116, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 148, 113, 113, 149, 150, 
	149, 149, 149, 149, 149, 149, 149, 149, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 140, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 140, 113, 
	151, 152, 151, 151, 151, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 153, 
	113, 114, 115, 114, 114, 114, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	116, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 154, 113, 113, 
	149, 150, 149, 149, 149, 149, 149, 149, 
	149, 149, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 140, 155, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	140, 113, 113, 155, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 155, 113, 151, 156, 151, 
	151, 151, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 153, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 157, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	157, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 157, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 157, 
	113, 61, 158, 61, 61, 61, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	62, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 93, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 93, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 93, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 93, 60, 61, 158, 61, 
	61, 61, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 119, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 93, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	93, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 93, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 93, 
	60, 3, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 159, 2, 2, 130, 
	160, 130, 130, 130, 130, 130, 130, 130, 
	130, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 123, 133, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 123, 
	2, 2, 133, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 133, 2, 161, 162, 161, 161, 
	161, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 161, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	135, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 135, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 135, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 135, 2, 
	3, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 163, 2, 2, 130, 130, 
	130, 130, 130, 130, 130, 130, 130, 130, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 123, 133, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 123, 2, 
	2, 133, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 133, 2, 164, 165, 164, 164, 164, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 164, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 135, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 135, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 135, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 135, 2, 3, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 166, 2, 2, 130, 130, 130, 
	130, 130, 130, 130, 130, 130, 130, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 133, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	133, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	133, 2, 167, 168, 167, 167, 167, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 167, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 135, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 135, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 135, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 135, 2, 3, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 169, 2, 2, 130, 130, 130, 130, 
	130, 130, 130, 130, 130, 130, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	133, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 133, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 133, 
	2, 131, 170, 131, 131, 131, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	131, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 135, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 135, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 135, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 135, 2, 72, 171, 72, 
	72, 72, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 72, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 75, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	75, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 75, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 75, 
	1, 172, 173, 172, 172, 172, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	172, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 75, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 75, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 75, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 75, 1, 174, 1, 1, 
	71, 71, 71, 71, 71, 71, 71, 71, 
	71, 71, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 73, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 73, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 73, 1, 175, 176, 175, 
	175, 175, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 175, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 75, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	75, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 75, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 75, 
	1, 177, 1, 1, 71, 71, 71, 71, 
	71, 71, 71, 71, 71, 71, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	73, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 73, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 73, 
	1, 178, 179, 178, 178, 178, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	178, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	130, 130, 130, 130, 130, 130, 130, 130, 
	130, 130, 2, 3, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 180, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 123, 133, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 123, 2, 2, 133, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 133, 2, 124, 181, 
	124, 124, 124, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 124, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 135, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 135, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	135, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	135, 2, 67, 182, 67, 67, 67, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 67, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 75, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 75, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 75, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 75, 1, 114, 183, 
	114, 114, 114, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 116, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 157, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 157, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	157, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	157, 113, 184, 185, 184, 184, 184, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 186, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 149, 149, 149, 149, 149, 149, 149, 
	149, 149, 149, 113, 114, 115, 114, 114, 
	114, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 116, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	187, 113, 113, 113, 117, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 155, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 155, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 155, 113, 
	136, 188, 136, 136, 136, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 138, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 157, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 157, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 157, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 157, 113, 3, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 189, 
	2, 2, 2, 120, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 123, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 123, 2, 190, 191, 190, 190, 
	190, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 190, 2, 3, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	192, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 123, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 123, 2, 193, 194, 193, 
	193, 193, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 193, 2, 3, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 195, 2, 2, 130, 130, 130, 130, 
	130, 130, 130, 130, 130, 130, 2, 167, 
	196, 167, 167, 167, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 167, 2, 
	172, 172, 172, 172, 172, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 172, 
	1, 197, 197, 197, 197, 197, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	197, 1, 198, 1, 1, 71, 71, 71, 
	71, 71, 71, 71, 71, 71, 71, 1, 
	3, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 199, 2, 2, 2, 120, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 133, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 133, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 133, 2, 121, 200, 121, 121, 121, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 121, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 135, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 135, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 135, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 135, 2, 201, 
	202, 201, 201, 201, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 203, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 149, 149, 
	149, 149, 149, 149, 149, 149, 149, 149, 
	113, 114, 115, 114, 114, 114, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	116, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 204, 113, 113, 
	113, 117, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 140, 155, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	140, 113, 113, 155, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 155, 113, 141, 205, 141, 
	141, 141, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 143, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 157, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	157, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 157, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 157, 
	113, 3, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 206, 2, 2, 2, 
	120, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 123, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 123, 
	2, 207, 208, 207, 207, 207, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	207, 2, 3, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 209, 2, 2, 
	130, 130, 130, 130, 130, 130, 130, 130, 
	130, 130, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 123, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	123, 2, 164, 210, 164, 164, 164, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 164, 2, 175, 175, 175, 175, 175, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 175, 1, 211, 212, 211, 211, 
	211, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 213, 113, 114, 115, 114, 
	114, 114, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 116, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 214, 113, 113, 113, 117, 113, 215, 
	216, 215, 215, 215, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 217, 113, 
	114, 115, 114, 114, 114, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 116, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 218, 113, 113, 149, 
	150, 149, 149, 149, 149, 149, 149, 149, 
	149, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 140, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 140, 
	113, 219, 220, 219, 219, 219, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	221, 113, 114, 115, 114, 114, 114, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 116, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 222, 113, 
	113, 113, 117, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 140, 155, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 140, 113, 113, 155, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 155, 113, 145, 223, 
	145, 145, 145, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 147, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 157, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	113, 157, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	157, 113, 113, 113, 113, 113, 113, 113, 
	113, 113, 113, 113, 113, 113, 113, 113, 
	157, 113, 3, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 224, 2, 2, 
	130, 160, 130, 130, 130, 130, 130, 130, 
	130, 130, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 123, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	123, 2, 161, 225, 161, 161, 161, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 161, 2, 3, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 226, 2, 
	2, 2, 120, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 123, 133, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 123, 2, 2, 133, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 133, 2, 207, 227, 
	207, 207, 207, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 207, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 135, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 135, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	135, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	135, 2, 3, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 228, 2, 2, 
	130, 160, 130, 130, 130, 130, 130, 130, 
	130, 130, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 123, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	123, 2, 229, 230, 229, 229, 229, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 229, 2, 3, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 231, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 123, 133, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 123, 2, 2, 133, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 133, 2, 193, 232, 
	193, 193, 193, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 193, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 135, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 135, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	135, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	135, 2, 197, 233, 197, 197, 197, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 197, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 75, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 75, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 75, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 75, 1, 3, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 234, 2, 2, 2, 120, 2, 235, 
	236, 235, 235, 235, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 235, 2, 
	3, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 237, 2, 2, 130, 130, 
	130, 130, 130, 130, 130, 130, 130, 130, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 123, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 123, 2, 
	238, 239, 238, 238, 238, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 238, 
	2, 3, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 240, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 133, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 133, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 133, 2, 127, 241, 127, 127, 
	127, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 127, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	135, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 135, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 135, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 135, 2, 
	69, 242, 69, 69, 69, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 69, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 75, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 75, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 75, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 75, 1, 243, 243, 243, 243, 
	243, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 243, 1, 244, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 73, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 73, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 73, 1, 3, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	245, 2, 2, 2, 120, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 123, 133, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 123, 2, 2, 133, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 133, 2, 
	190, 246, 190, 190, 190, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 190, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 135, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 135, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 135, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 135, 2, 247, 1, 1, 1, 
	64, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 66, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 66, 
	1, 248, 248, 248, 248, 248, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	248, 1, 249, 1, 1, 71, 71, 71, 
	71, 71, 71, 71, 71, 71, 71, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	66, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 66, 1, 250, 
	1, 1, 1, 64, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 66, 73, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 66, 1, 1, 73, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 73, 1, 251, 
	252, 251, 251, 251, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 251, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 75, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 75, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 75, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 75, 1, 253, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 66, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 66, 1, 
	254, 1, 1, 71, 255, 71, 71, 71, 
	71, 71, 71, 71, 71, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 66, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 66, 1, 256, 256, 256, 
	256, 256, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 256, 1, 257, 1, 
	1, 71, 71, 71, 71, 71, 71, 71, 
	71, 71, 71, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 66, 73, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 66, 1, 1, 73, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 73, 1, 258, 258, 
	258, 258, 258, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 258, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 71, 71, 71, 
	71, 71, 71, 71, 71, 71, 71, 1, 
	259, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 66, 73, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 66, 1, 1, 73, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 73, 1, 
	260, 1, 1, 1, 64, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 66, 73, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 66, 1, 1, 73, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 73, 1, 
	248, 261, 248, 248, 248, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 248, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 75, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 75, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 75, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 75, 1, 262, 1, 1, 71, 
	255, 71, 71, 71, 71, 71, 71, 71, 
	71, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 66, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 66, 
	1, 263, 263, 263, 263, 263, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	263, 1, 264, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	66, 73, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 66, 1, 1, 
	73, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	73, 1, 265, 1, 1, 1, 64, 1, 
	266, 266, 266, 266, 266, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 266, 
	1, 267, 1, 1, 71, 71, 71, 71, 
	71, 71, 71, 71, 71, 71, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 66, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 66, 1, 268, 1, 
	1, 1, 64, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 66, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 66, 1, 251, 251, 251, 251, 251, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 251, 1, 269, 1, 1, 1, 
	64, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 73, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 73, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 73, 1, 65, 270, 65, 65, 
	65, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 65, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	75, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 75, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 75, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 75, 1, 
	271, 1, 1, 71, 255, 71, 71, 71, 
	71, 71, 71, 71, 71, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 66, 73, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 66, 1, 1, 73, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 73, 1, 
	256, 272, 256, 256, 256, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 256, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 75, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 75, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 75, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 75, 1, 59, 59, 59, 59, 
	59, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 59, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 58, 58, 58, 58, 
	58, 58, 58, 58, 58, 58, 58, 58, 
	58, 58, 58, 58, 58, 58, 58, 58, 
	58, 58, 58, 58, 58, 58, 1, 1, 
	1, 1, 1, 1, 58, 58, 58, 58, 
	58, 58, 58, 58, 58, 58, 58, 58, 
	58, 58, 58, 58, 58, 58, 58, 58, 
	58, 58, 58, 58, 58, 58, 1, 273, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 274, 1, 1, 1, 275, 1, 276, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 277, 1, 1, 1, 278, 1, 61, 
	61, 61, 61, 61, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 62, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 90, 60, 60, 86, 87, 
	86, 86, 86, 86, 86, 86, 86, 86, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 79, 91, 279, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 280, 79, 60, 
	60, 281, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 91, 60, 61, 61, 61, 61, 61, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 62, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 78, 
	60, 60, 60, 63, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 79, 60, 279, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 280, 79, 60, 60, 282, 60, 61, 
	61, 61, 61, 61, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 62, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 85, 60, 60, 86, 87, 
	86, 86, 86, 86, 86, 86, 86, 86, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 79, 60, 279, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 280, 79, 60, 
	60, 282, 60, 61, 61, 61, 61, 61, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 62, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 82, 
	60, 60, 60, 63, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 79, 60, 279, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 280, 79, 60, 60, 282, 60, 61, 
	61, 61, 61, 61, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 62, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 63, 60, 60, 60, 63, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 283, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 284, 60, 60, 
	60, 285, 60, 73, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 73, 276, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 277, 1, 1, 
	1, 286, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 73, 1, 70, 1, 1, 71, 71, 
	71, 71, 71, 71, 71, 71, 71, 71, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 276, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 277, 1, 1, 
	1, 278, 1, 68, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 276, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 277, 1, 1, 
	1, 278, 1, 66, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 66, 1, 276, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 277, 66, 1, 
	1, 278, 1, 61, 61, 61, 61, 61, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 62, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 90, 
	60, 60, 86, 87, 86, 86, 86, 86, 
	86, 86, 86, 86, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 79, 91, 283, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 284, 79, 60, 60, 287, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 91, 60, 61, 
	61, 61, 61, 61, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 62, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 63, 60, 60, 60, 63, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 279, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 280, 60, 60, 
	60, 282, 60, 174, 1, 1, 71, 71, 
	71, 71, 71, 71, 71, 71, 71, 71, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 73, 276, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 277, 1, 1, 
	1, 286, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 73, 1, 73, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 73, 273, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 274, 1, 1, 
	1, 288, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 73, 1, 177, 1, 1, 71, 71, 
	71, 71, 71, 71, 71, 71, 71, 71, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 73, 276, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 277, 1, 1, 
	1, 286, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 73, 1, 174, 1, 1, 71, 71, 
	71, 71, 71, 71, 71, 71, 71, 71, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 73, 273, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 274, 1, 1, 
	1, 288, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 73, 1, 257, 1, 1, 71, 71, 
	71, 71, 71, 71, 71, 71, 71, 71, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 66, 73, 276, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 277, 66, 1, 
	1, 286, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 73, 1, 177, 1, 1, 71, 71, 
	71, 71, 71, 71, 71, 71, 71, 71, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 73, 273, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 274, 1, 1, 
	1, 288, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 73, 1, 259, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 66, 73, 276, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 277, 66, 1, 
	1, 286, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 73, 1, 68, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 273, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 274, 1, 1, 
	1, 275, 1, 61, 61, 61, 61, 61, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 62, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 78, 
	60, 60, 60, 63, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 79, 60, 283, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 284, 79, 60, 60, 285, 60, 198, 
	1, 1, 71, 71, 71, 71, 71, 71, 
	71, 71, 71, 71, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 276, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 277, 1, 1, 1, 278, 1, 253, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 66, 1, 276, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 277, 66, 1, 1, 278, 1, 61, 
	61, 61, 61, 61, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 62, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 96, 60, 60, 60, 63, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 91, 283, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 284, 60, 60, 
	60, 287, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 91, 60, 61, 61, 61, 61, 61, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 62, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 82, 
	60, 60, 60, 63, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 79, 60, 283, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 284, 79, 60, 60, 285, 60, 249, 
	1, 1, 71, 71, 71, 71, 71, 71, 
	71, 71, 71, 71, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 66, 1, 276, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 277, 66, 1, 1, 278, 1, 61, 
	61, 61, 61, 61, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 62, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 85, 60, 60, 86, 87, 
	86, 86, 86, 86, 86, 86, 86, 86, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 79, 60, 283, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 284, 79, 60, 
	60, 285, 60, 61, 61, 61, 61, 61, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 62, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 106, 
	60, 60, 60, 63, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 79, 91, 283, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 284, 79, 60, 60, 287, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 91, 60, 61, 
	61, 61, 61, 61, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 62, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 103, 60, 60, 86, 87, 
	86, 86, 86, 86, 86, 86, 86, 86, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 79, 60, 283, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 284, 79, 60, 
	60, 285, 60, 264, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 66, 73, 276, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 277, 66, 1, 
	1, 286, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 73, 1, 198, 1, 1, 71, 71, 
	71, 71, 71, 71, 71, 71, 71, 71, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 273, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 274, 1, 1, 
	1, 275, 1, 61, 61, 61, 61, 61, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 62, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 100, 
	60, 60, 60, 63, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 283, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 284, 60, 60, 60, 285, 60, 244, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 73, 276, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 277, 1, 1, 1, 286, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 73, 1, 70, 
	1, 1, 71, 71, 71, 71, 71, 71, 
	71, 71, 71, 71, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 273, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 274, 1, 1, 1, 275, 1, 267, 
	1, 1, 71, 71, 71, 71, 71, 71, 
	71, 71, 71, 71, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 66, 1, 276, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 277, 66, 1, 1, 278, 1, 61, 
	61, 61, 61, 61, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 62, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 110, 60, 60, 60, 63, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 79, 91, 283, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 284, 79, 60, 
	60, 287, 60, 60, 60, 60, 60, 60, 
	60, 60, 60, 60, 60, 60, 60, 60, 
	60, 91, 60, 253, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 66, 1, 273, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 274, 66, 1, 
	1, 275, 1, 249, 1, 1, 71, 71, 
	71, 71, 71, 71, 71, 71, 71, 71, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 66, 1, 273, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 274, 66, 1, 
	1, 275, 1, 66, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 66, 1, 273, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 274, 66, 1, 
	1, 275, 1, 257, 1, 1, 71, 71, 
	71, 71, 71, 71, 71, 71, 71, 71, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 66, 73, 273, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 274, 66, 1, 
	1, 288, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 73, 1, 0
};

static const short _keyinfo_trans_targs[] = {
	2, 0, 3, 221, 5, 7, 8, 9, 
	10, 11, 12, 13, 14, 15, 16, 17, 
	18, 19, 20, 21, 22, 23, 24, 25, 
	26, 27, 28, 29, 30, 31, 32, 33, 
	34, 35, 36, 37, 38, 39, 40, 41, 
	42, 43, 44, 45, 46, 47, 48, 49, 
	50, 51, 52, 53, 54, 55, 56, 57, 
	58, 59, 219, 60, 61, 62, 63, 74, 
	64, 65, 66, 67, 68, 69, 70, 73, 
	71, 72, 220, 72, 75, 213, 76, 86, 
	77, 194, 78, 79, 200, 80, 83, 93, 
	81, 217, 82, 125, 222, 125, 84, 215, 
	85, 223, 87, 210, 88, 89, 207, 90, 
	91, 205, 92, 224, 94, 197, 95, 225, 
	97, 98, 99, 226, 104, 116, 101, 103, 
	105, 106, 230, 107, 108, 229, 109, 110, 
	228, 111, 115, 112, 227, 113, 221, 113, 
	117, 241, 148, 118, 167, 119, 245, 162, 
	120, 121, 247, 174, 122, 145, 159, 123, 
	231, 127, 124, 144, 231, 144, 232, 128, 
	140, 129, 237, 130, 131, 235, 132, 133, 
	233, 134, 227, 234, 137, 236, 135, 139, 
	238, 136, 141, 239, 142, 229, 240, 226, 
	146, 244, 157, 147, 241, 149, 150, 243, 
	151, 152, 242, 153, 233, 156, 154, 158, 
	230, 160, 256, 192, 161, 245, 163, 164, 
	246, 165, 235, 168, 252, 183, 169, 170, 
	249, 178, 171, 172, 248, 176, 173, 247, 
	175, 237, 177, 246, 179, 180, 250, 181, 
	242, 251, 184, 185, 255, 186, 187, 253, 
	188, 228, 254, 191, 189, 193, 243, 195, 
	196, 166, 198, 199, 257, 155, 201, 203, 
	202, 138, 204, 143, 206, 258, 208, 209, 
	182, 211, 212, 190, 214, 216, 259, 218, 
	260, 1, 4, 6, 1, 4, 6, 96, 
	100, 126, 102, 96, 100, 102, 114, 126, 
	114
};

static const char _keyinfo_trans_actions[] = {
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 11, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 5, 3, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 5, 5, 3, 0, 0, 0, 
	5, 3, 0, 0, 0, 0, 0, 0, 
	0, 0, 5, 3, 0, 0, 5, 3, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 5, 3, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 5, 5, 3, 0, 3, 5, 
	0, 0, 3, 5, 0, 3, 5, 0, 
	3, 5, 3, 3, 0, 3, 5, 0, 
	3, 5, 0, 0, 5, 3, 3, 3, 
	0, 0, 0, 5, 3, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 5, 
	3, 0, 0, 0, 5, 3, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 5, 3, 
	0, 0, 5, 3, 0, 0, 0, 5, 
	3, 3, 0, 0, 0, 0, 0, 0, 
	5, 3, 3, 0, 5, 5, 3, 0, 
	0, 0, 5, 0, 3, 0, 0, 0, 
	0, 5, 0, 5, 5, 3, 0, 0, 
	5, 0, 0, 0, 0, 5, 3, 5, 
	3, 7, 0, 0, 17, 9, 9, 7, 
	0, 5, 0, 17, 9, 9, 14, 14, 
	5
};

static const char _keyinfo_eof_actions[] = {
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 9, 0, 0, 
	0, 0, 9, 9, 9, 9, 9, 9, 
	0, 9, 0, 9, 0, 9, 0, 9, 
	0, 9, 9, 9, 9, 9, 9, 9, 
	9, 9, 9, 0, 9, 9, 0, 9, 
	9, 0, 0, 0, 0
};

static const int keyinfo_start = 220;
static const int keyinfo_error = 0;

static const int keyinfo_en_main = 220;


#line 275 "src/gpg-protocol.rl"

struct fingerprint_t *gpg_keyinfo(struct gpg_t *gpg)
{
    static const char message[] = "KEYINFO --list --with-ssh\n";
    struct fingerprint_t *fpt = NULL;
    char keygrip[40];
    size_t keylen = 0;
    enum keyflags keyflags = 0;

    ssize_t nbytes_w = write(gpg->fd, message, sizeof(message) - 1);
    if (nbytes_w < 0)
        return NULL;

    
#line 289 "src/gpg-protocol.rl"
    
#line 290 "src/gpg-protocol.rl"
    
#line 291 "src/gpg-protocol.rl"
    
#line 2329 "src/gpg-protocol.c"
	{
	 gpg->cs = keyinfo_start;
	}

#line 292 "src/gpg-protocol.rl"

    for (;;) {
        if (gpg->p == NULL || gpg->p == gpg->pe) {
            if (gpg_buffer_refill(gpg) <= 0)
                break;
        }

        char *eof = gpg->pe;
        
#line 2344 "src/gpg-protocol.c"
	{
	int _slen;
	int _trans;
	const char *_acts;
	unsigned int _nacts;
	const char *_keys;
	const short *_inds;

	if ( ( gpg->p) == ( gpg->pe) )
		goto _test_eof;
	if (  gpg->cs == 0 )
		goto _out;
_resume:
	_keys = _keyinfo_trans_keys + ( gpg->cs<<1);
	_inds = _keyinfo_indicies + _keyinfo_index_offsets[ gpg->cs];

	_slen = _keyinfo_key_spans[ gpg->cs];
	_trans = _inds[ _slen > 0 && _keys[0] <=(*( gpg->p)) &&
		(*( gpg->p)) <= _keys[1] ?
		(*( gpg->p)) - _keys[0] : _slen ];

	 gpg->cs = _keyinfo_trans_targs[_trans];

	if ( _keyinfo_trans_actions[_trans] == 0 )
		goto _again;

	_acts = _keyinfo_actions + _keyinfo_trans_actions[_trans];
	_nacts = (unsigned int) *_acts++;
	while ( _nacts-- > 0 ) {
		switch ( *(_acts++) )
		{
	case 0:
#line 189 "src/gpg-protocol.rl"
	{ keylen = 0; }
	break;
	case 1:
#line 190 "src/gpg-protocol.rl"
	{ keygrip[keylen++] = (*( gpg->p)); }
	break;
	case 2:
#line 191 "src/gpg-protocol.rl"
	{
        struct fingerprint_t *next = fpt;
        fpt = malloc(sizeof(struct fingerprint_t));
        *fpt = (struct fingerprint_t){
            .fingerprint = strndup(keygrip, keylen),
            .flags = keyflags,
            .next = next
        };
        keyflags = 0;
    }
	break;
	case 3:
#line 202 "src/gpg-protocol.rl"
	{
        switch ((*( gpg->p))) {
        case 'D':
            keyflags |= KEY_DISABLED;
            break;
        case 'S':
            keyflags |= KEY_SSHCONTROL;
            break;
        case 'c':
            keyflags |= KEY_CONFIRM;
            break;
        }
    }
	break;
	case 4:
#line 216 "src/gpg-protocol.rl"
	{ fprintf(stderr, "%s: gpg protocol error: %s", program_invocation_short_name, ( gpg->p)); }
	break;
	case 5:
#line 217 "src/gpg-protocol.rl"
	{ return fpt; }
	break;
#line 2421 "src/gpg-protocol.c"
		}
	}

_again:
	if (  gpg->cs == 0 )
		goto _out;
	if ( ++( gpg->p) != ( gpg->pe) )
		goto _resume;
	_test_eof: {}
	if ( ( gpg->p) == eof )
	{
	const char *__acts = _keyinfo_actions + _keyinfo_eof_actions[ gpg->cs];
	unsigned int __nacts = (unsigned int) *__acts++;
	while ( __nacts-- > 0 ) {
		switch ( *__acts++ ) {
	case 5:
#line 217 "src/gpg-protocol.rl"
	{ return fpt; }
	break;
#line 2441 "src/gpg-protocol.c"
		}
	}
	}

	_out: {}
	}

#line 301 "src/gpg-protocol.rl"

        if (gpg->cs == keyinfo_error) {
            warnx("error parsing gpg protocol");
            break;
        }
    }

    (void)keyinfo_en_main;
    return fpt;
}

int gpg_preset_passphrase(struct gpg_t *gpg, const char *fingerprint, int timeout, const char *password)
{
    static const char *hex_digits = "0123456789ABCDEF";

    if (!fingerprint)
        return -EINVAL;

    if (!password)
        return gpg_send_message(gpg, "PRESET_PASSPHRASE %s %d\n", fingerprint, timeout);

    size_t i, size = strlen(password);
    char bin_password[2 * size + 1];

    for(i = 0; i < size; i++) {
        bin_password[2 * i] = hex_digits[password[i] >> 4];
        bin_password[2 * i + 1] = hex_digits[password[i] & 0x0f];
    }

    bin_password[2 * size] = '\0';
    return gpg_send_message(gpg, "PRESET_PASSPHRASE %s %d %s\n", fingerprint, timeout, bin_password);
}

void free_fingerprints(struct fingerprint_t *fpt)
{
    while (fpt) {
        struct fingerprint_t *node = fpt;
        fpt = fpt->next;

        free(node->fingerprint);
        free(node);
    }
}

void gpg_close(struct gpg_t *gpg)
{
    close(gpg->fd);
    free(gpg);
}

// vim: et:sts=4:sw=4:cino=(0
